"""Subclass of settings_dialog, which is generated by wxFormBuilder."""
import re
import os
import wx
import wx.grid
import pcbnew

from . import dialog_base


def pop_error(msg):
    wx.MessageBox(msg, 'Error', wx.OK | wx.ICON_ERROR)

class SettingsDialog(dialog_base.SettingsDialogBase):
    def __init__(self, config, perform_export_func, load_saved_func, version):
        dialog_base.SettingsDialogBase.__init__(self, None)
        self.panel = SettingsDialogPanel(self, config, perform_export_func, load_saved_func)
        best_size = self.panel.BestSize
        # hack for some gtk themes that incorrectly calculate best size
        best_size.IncBy(dx=0, dy=30)
        self.SetClientSize(best_size)
        self.SetTitle('Board2Pdf %s' % version)
        if (int(pcbnew.Version()[0:1]) < 8):
            # If KiCad version < 8, no support for exporting property popups on just Front or Back layer
            pos = self.panel.m_comboBox_popups.FindString("Front Layer")
            self.panel.m_comboBox_popups.Delete(pos)
            pos = self.panel.m_comboBox_popups.FindString("Back Layer")
            self.panel.m_comboBox_popups.Delete(pos)

    # hack for new wxFormBuilder generating code incompatible with old wxPython
    # noinspection PyMethodOverriding
    def SetSizeHints(self, sz1, sz2):
        try:
            # wxPython 4
            super(SettingsDialog, self).SetSizeHints(sz1, sz2)
        except TypeError:
            # wxPython 3
            self.SetSizeHintsSz(sz1, sz2)

# Implementing settings_dialog
class SettingsDialogPanel(dialog_base.SettingsDialogPanel):
    def __init__(self, parent, config, perform_export_func, load_saved_func):
        self.config = config
        self.perform_export_func = perform_export_func
        self.load_saved_func = load_saved_func
        self.templates: dict = config.templates
        self.current_template: str = ""
        self.current_layer: str = ""
        dialog_base.SettingsDialogPanel.__init__(self, parent)
        self.m_color_shower.SetBackgroundColour(wx.NullColour)
        self.m_color_shower.SetForegroundColour(wx.NullColour)
        self.m_color_shower.SetLabel("")
        self.hide_template_settings()
        self.hide_layer_settings()

        self.save_menu = wx.Menu()
        self.save_globally = self.save_menu.Append(
            wx.ID_ANY, u"Globally", wx.EmptyString, wx.ITEM_NORMAL)
        self.save_locally = self.save_menu.Append(
            wx.ID_ANY, u"Locally", wx.EmptyString, wx.ITEM_NORMAL)

        self.Bind(
            wx.EVT_MENU, self.OnSaveGlobally, id=self.save_globally.GetId())
        self.Bind(
            wx.EVT_MENU, self.OnSaveLocally, id=self.save_locally.GetId())

        self.load_menu = wx.Menu()
        self.load_default = self.load_menu.Append(
            wx.ID_ANY, u"Default", wx.EmptyString, wx.ITEM_NORMAL)
        self.load_globally = self.load_menu.Append(
            wx.ID_ANY, u"Globally", wx.EmptyString, wx.ITEM_NORMAL)
        self.load_locally = self.load_menu.Append(
            wx.ID_ANY, u"Locally", wx.EmptyString, wx.ITEM_NORMAL)

        self.Bind(
            wx.EVT_MENU, self.OnLoadDefault, id=self.load_default.GetId())
        self.Bind(
            wx.EVT_MENU, self.OnLoadGlobal, id=self.load_globally.GetId())
        self.Bind(
            wx.EVT_MENU, self.OnLoadLocal, id=self.load_locally.GetId())

    def OnLoadSettings(self, event):
        # type: (wx.CommandEvent) -> None
        pos = wx.Point(0, event.GetEventObject().GetSize().y)
        self.loadSettingsBtn.PopupMenu(self.load_menu, pos)

    def OnLoadDefault(self, event):
        self.config._configfile = self.config.default_settings_file_path
        self.config.load()
        self.load_saved_func(self, self.config)
        self.m_staticText_status.SetLabel('Status: default settings loaded')

    def OnLoadGlobal(self, event):
        self.config._configfile = self.config.global_settings_file_path
        self.config.load()
        self.load_saved_func(self, self.config)
        self.m_staticText_status.SetLabel('Status: global settings loaded')

    def OnLoadLocal(self, event):
        self.config._configfile = self.config.local_settings_file_path
        self.config.load()
        self.load_saved_func(self, self.config)
        self.m_staticText_status.SetLabel('Status: local settings loaded')

    def OnSaveSettings(self, event):
        # type: (wx.CommandEvent) -> None
        pos = wx.Point(0, event.GetEventObject().GetSize().y)
        self.saveSettingsBtn.PopupMenu(self.save_menu, pos)

    def OnSaveGlobally(self, event):
        self.SaveTemplate()

        self.config.output_path = self.outputDirPicker.Path
        self.config.enabled_templates = self.templatesSortOrderBox.GetItems()
        self.config.disabled_templates = self.disabledTemplatesSortOrderBox.GetItems()
        self.config.create_svg = self.m_checkBox_create_svg.IsChecked()
        self.config.del_temp_files = self.m_checkBox_delete_temp_files.IsChecked()
        self.config.del_single_page_files = self.m_checkBox_delete_single_page_files.IsChecked()
        self.config.save(self.config.global_settings_file_path)

        self.m_staticText_status.SetLabel('Status: settings saved globally')

    def OnSaveLocally(self, event):
        self.SaveTemplate()

        self.config.output_path = self.outputDirPicker.Path
        self.config.enabled_templates = self.templatesSortOrderBox.GetItems()
        self.config.disabled_templates = self.disabledTemplatesSortOrderBox.GetItems()
        self.config.create_svg = self.m_checkBox_create_svg.IsChecked()
        self.config.del_temp_files = self.m_checkBox_delete_temp_files.IsChecked()
        self.config.del_single_page_files = self.m_checkBox_delete_single_page_files.IsChecked()
        self.config.save(self.config.local_settings_file_path)

        self.m_staticText_status.SetLabel('Status: settings saved locally')

    def hide_template_settings(self):
        self.m_textCtrl_template_name.Disable()
        self.m_comboBox_frame.Disable()
        self.m_checkBox_mirror.Disable()
        self.m_checkBox_tent.Disable()
        self.m_staticText_template_name.Disable()
        self.m_staticText_frame_layer.Disable()
        self.m_staticText_popups.Disable()
        self.m_comboBox_popups.Disable()

        self.layersSortOrderBox.Disable()
        self.m_button_layer_up.Disable()
        self.m_button_layer_down.Disable()
        self.m_button_layer_disable.Disable()
        self.disabledLayersSortOrderBox.Disable()
        self.m_button_layer_enable.Disable()
        self.m_staticText_layer_info.Disable()

    def hide_layer_settings(self):
        self.m_textCtrl_color.Disable()
        self.m_button_pick_color.Disable()
        self.m_checkBox_negative.Disable()
        self.m_checkBox_reference_designators.Disable()
        self.m_checkBox_footprint_values.Disable()
        self.m_staticText_layer_color.Disable()
        self.m_textCtrl_color.ChangeValue("")
        self.m_color_shower.SetBackgroundColour(wx.NullColour)
        self.m_color_shower.SetForegroundColour(wx.NullColour)
        self.m_color_shower.SetLabel("")
        self.m_checkBox_negative.SetValue(False)
        self.m_checkBox_footprint_values.SetValue(True)
        self.m_checkBox_reference_designators.SetValue(True)

    def show_template_settings(self):
        self.m_textCtrl_template_name.Enable()
        self.m_comboBox_frame.Enable()
        self.m_checkBox_mirror.Enable()
        self.m_checkBox_tent.Enable()
        self.m_staticText_template_name.Enable()
        self.m_staticText_frame_layer.Enable()
        self.m_staticText_popups.Enable()
        self.m_comboBox_popups.Enable()

        self.layersSortOrderBox.Enable()
        self.m_button_layer_up.Enable()
        self.m_button_layer_down.Enable()
        self.m_button_layer_disable.Enable()
        self.disabledLayersSortOrderBox.Enable()
        self.m_button_layer_enable.Enable()
        self.m_staticText_layer_info.Enable()

    def show_layer_settings(self):
        self.m_textCtrl_color.Enable()
        self.m_button_pick_color.Enable()
        self.m_checkBox_negative.Enable()
        self.m_checkBox_reference_designators.Enable()
        self.m_checkBox_footprint_values.Enable()
        self.m_staticText_layer_color.Enable()

    def OnExit(self, event):
        self.GetParent().EndModal(wx.ID_CANCEL)

    def OnPerform(self, event):
        self.SaveTemplate()
        self.perform_export_func(self)

    def finish_init(self):
        print("finish_init()")

    # Handlers for events.
    def OnTemplateSortOrderUp(self, event):
        selection = self.templatesSortOrderBox.Selection
        if selection != wx.NOT_FOUND and selection > 0:
            item = self.templatesSortOrderBox.GetString(selection)
            self.templatesSortOrderBox.Delete(selection)
            self.templatesSortOrderBox.Insert(item, selection - 1)
            self.templatesSortOrderBox.SetSelection(selection - 1)

    def OnTemplateSortOrderDown(self, event):
        selection = self.templatesSortOrderBox.Selection
        size = self.templatesSortOrderBox.Count
        if selection != wx.NOT_FOUND and selection < size - 1:
            item = self.templatesSortOrderBox.GetString(selection)
            self.templatesSortOrderBox.Delete(selection)
            self.templatesSortOrderBox.Insert(item, selection + 1)
            self.templatesSortOrderBox.SetSelection(selection + 1)

    def OnTemplateNew(self, event):
        self.SaveTemplate()
        item = wx.GetTextFromUser(
            "Characters except for A-Z, a-z, 0-9, -, +, _ and ' ' will be ignored.",
            "Add new template")
        item = re.sub('[^A-Za-z0-9\-\+ _]', '', item)
        if item == '':
            return

        found_en = self.templatesSortOrderBox.FindString(item)
        found_dis = self.disabledTemplatesSortOrderBox.FindString(item)
        if found_en != wx.NOT_FOUND or found_dis != wx.NOT_FOUND:
            wx.MessageBox("The template name '" + item + "' already exists", "Error", wx.ICON_ERROR)
            return

        self.templatesSortOrderBox.Append(item)
        self.templatesSortOrderBox.SetSelection(
            self.templatesSortOrderBox.Count - 1)
        self.ClearTemplateSettings()
        self.OnTemplateEdit(event)

    def OnTemplateClone(self, event):
        self.SaveTemplate()

        selection = self.templatesSortOrderBox.Selection
        if selection != wx.NOT_FOUND:
            item = self.templatesSortOrderBox.GetString(selection)

            new_item = item + "-Copy"
            found_en = self.templatesSortOrderBox.FindString(new_item)
            found_dis = self.disabledTemplatesSortOrderBox.FindString(new_item)
            if found_en != wx.NOT_FOUND or found_dis != wx.NOT_FOUND:
                wx.MessageBox("The template name '" + new_item + "' already exists", "Error", wx.ICON_ERROR)
                return

            if self.current_template == '':
                return
            self.templates[new_item] = self.templates[self.current_template]
            self.templatesSortOrderBox.Append(new_item)
            self.templatesSortOrderBox.SetSelection(
                self.templatesSortOrderBox.Count - 1)
            # wx.MessageBox("Created " + new_item + " as a clone of " + item)
            self.OnTemplateEdit(event)
            self.current_template = new_item
            self.SaveTemplate()

    @staticmethod
    def _listbox_reselect(listbox, selection):
        if listbox.Count > 0:
            if listbox.Count <= max(selection, 0):
                listbox.SetSelection(max(selection - 1, 0))
            else:
                listbox.SetSelection(max(selection, 0))
            return True
        return False

    def OnTemplateDisable(self, event):
        self.SaveTemplate()
        selection = self.templatesSortOrderBox.Selection
        if selection != wx.NOT_FOUND:
            item = self.templatesSortOrderBox.GetString(selection)
            self.templatesSortOrderBox.Delete(selection)
            self.disabledTemplatesSortOrderBox.Append(item)
            self._listbox_reselect(self.templatesSortOrderBox, selection)

            self.ClearTemplateSettings()
            self.hide_template_settings()
            self.OnTemplateEdit(event)

    def OnTemplateEnable(self, event):
        selection = self.disabledTemplatesSortOrderBox.Selection
        if selection != wx.NOT_FOUND:
            item = self.disabledTemplatesSortOrderBox.GetString(selection)
            self.disabledTemplatesSortOrderBox.Delete(selection)
            self.templatesSortOrderBox.Append(item)
            self._listbox_reselect(self.disabledTemplatesSortOrderBox, selection)

    def OnTemplateDelete(self, event):
        selection = self.disabledTemplatesSortOrderBox.Selection
        if selection != wx.NOT_FOUND:
            item = self.disabledTemplatesSortOrderBox.GetString(selection)
            del self.templates[item]
            self.disabledTemplatesSortOrderBox.Delete(selection)
            self._listbox_reselect(self.disabledTemplatesSortOrderBox, selection)

    def OnPickColor(self, event):
        _rgbstring = re.compile(r'#[a-fA-F0-9]{3}(?:[a-fA-F0-9]{3})?$')
        color_value = self.m_textCtrl_color.GetValue()
        if self.current_layer == "":
            wx.MessageBox("You must first select a layer.")
        else:
            if not bool(_rgbstring.match(color_value)):
                color_value = "#000000"

            value = color_value.lstrip('#')
            lv = len(value)
            rgb = tuple(int(value[i:i + lv // 3], 16) for i in range(0, lv, lv // 3))
            rgb = wx.Colour(rgb[0], rgb[1], rgb[2], 255)

            data = wx.ColourData()
            data.SetChooseFull(True)

            # set the first custom color (index 0) to the last used color
            data.SetCustomColour(0, rgb)

            # set the default color in the chooser to the last used color
            data.SetColour(rgb)

            cd = wx.ColourDialog(self, data)
            # cd.GetColourData().SetChooseFull(True)

            if cd.ShowModal() == wx.ID_OK:
                rgb_color = cd.GetColourData().Colour[:3]
                self.m_textCtrl_color.ChangeValue(str('#%02X%02X%02X' % rgb_color))
                self.m_color_shower.SetBackgroundColour(rgb_color)
                self.m_color_shower.SetForegroundColour(rgb_color)
                self.m_color_shower.SetLabel(str(rgb_color))

            cd.Destroy()
            self.OnSaveLayer(event)

    def OnLayerSortOrderUp(self, event):
        selection = self.layersSortOrderBox.Selection
        if selection != wx.NOT_FOUND and selection > 0:
            item = self.layersSortOrderBox.GetString(selection)
            self.layersSortOrderBox.Delete(selection)
            self.layersSortOrderBox.Insert(item, selection - 1)
            self.layersSortOrderBox.SetSelection(selection - 1)

    def OnLayerSortOrderDown(self, event):
        selection = self.layersSortOrderBox.Selection
        size = self.layersSortOrderBox.Count
        if selection != wx.NOT_FOUND and selection < size - 1:
            item = self.layersSortOrderBox.GetString(selection)
            self.layersSortOrderBox.Delete(selection)
            self.layersSortOrderBox.Insert(item, selection + 1)
            self.layersSortOrderBox.SetSelection(selection + 1)

    def OnLayerDisable(self, event):
        selection = self.layersSortOrderBox.Selection
        if selection != wx.NOT_FOUND:
            item = self.layersSortOrderBox.GetString(selection)
            if item == self.m_comboBox_frame.GetValue():
                wx.MessageBox(
                    f"You cannot disable {item} if it's selected in the 'Draw frame on layer' setting. First change this setting.")
            else:
                self.layersSortOrderBox.Delete(selection)
                self.disabledLayersSortOrderBox.Append(item)
                if self._listbox_reselect(self.layersSortOrderBox, selection):
                    self.OnLayerEdit(self)
                else:
                    self.hide_layer_settings()
                self.SaveTemplate()

    def OnLayerEnable(self, event):
        selection = self.disabledLayersSortOrderBox.Selection
        if selection != wx.NOT_FOUND:
            item = self.disabledLayersSortOrderBox.GetString(selection)
            self.disabledLayersSortOrderBox.Delete(selection)
            self.layersSortOrderBox.Append(item)
            self._listbox_reselect(self.disabledLayersSortOrderBox, selection)
            self.SaveTemplate()

    def OnTemplateEdit(self, event):
        self.OnSaveLayer(self)
        self.SaveTemplate()

        selection = self.templatesSortOrderBox.Selection
        if selection != wx.NOT_FOUND:
            self.ClearTemplateSettings()
            self.show_template_settings()

            item = self.templatesSortOrderBox.GetString(selection)
            self.m_textCtrl_template_name.ChangeValue(item)
            self.current_template = item

            board = pcbnew.GetBoard()
            layers = []
            layers_dict = dict()
            i = pcbnew.PCBNEW_LAYER_ID_START
            while i < pcbnew.PCBNEW_LAYER_ID_START + pcbnew.PCB_LAYER_ID_COUNT:
                layer_std_name = pcbnew.BOARD.GetStandardLayerName(i)
                layer_name = pcbnew.BOARD.GetLayerName(board, i)
                layers_dict[layer_std_name] = layer_name
                if layer_std_name == layer_name:
                    layers.append(layer_name)
                else:
                    layers.append(layer_std_name + " (" + layer_name + ")")
                i += 1

            # Set enabled layers, if there are any in this template already
            if item in self.templates and "enabled_layers" in self.templates[item]:
                enabled_layers = self.templates[item]["enabled_layers"].split(',')
                enabled_layers = [l for l in enabled_layers if l != '']  # removes empty entries
                # Add the layer name within parenthesis if the layer name is not the standard layer name
                for i, layer_name in enumerate(enabled_layers):
                    if layers_dict[layer_name] != layer_name:
                        enabled_layers[i] = layer_name + " (" + layers_dict[layer_name] + ")"
                # Add all enabled layers to the enabled layers sort box.
                if enabled_layers:
                    self.layersSortOrderBox.SetItems(enabled_layers)

            # Update the listbox with disabled layers
            # Add all layers not in the enabled list
            for l in layers:
                # Remove the name within the parenthesis if there is one, before searching for the name
                layer_name = l.split(' (')[0]
                if self.layersSortOrderBox.FindString(layer_name) == wx.NOT_FOUND:
                    self.disabledLayersSortOrderBox.Append(l)

            # Create dictionary with all layers and their settings
            if item in self.templates:
                self.layersColorDict = self.templates[item].get("layers", {})
                self.layersNegativeDict = self.templates[item].get("layers_negative", {})
                self.layersFootprintValuesDict = self.templates[item].get("layers_footprint_values", {})
                self.layersReferenceDesignatorsDict = self.templates[item].get("layers_reference_designators", {})
            else:
                self.layersColorDict = {}
                self.layersNegativeDict = {}
                self.layersFootprintValuesDict = {}
                self.layersReferenceDesignatorsDict = {}

            # Update the comboBox where user can select one layer to plot the "frame"
            layers.insert(0, "None")
            self.m_comboBox_frame.SetItems(layers)
            self.m_comboBox_frame.SetSelection(0)
            # Check the saved template to see if there is a saved choice
            if item in self.templates and "frame" in self.templates[item]:
                saved_frame = self.templates[item]["frame"]
                if saved_frame and saved_frame != "None":
                    if layers_dict[saved_frame] != saved_frame:
                        saved_frame += " (" + layers_dict[saved_frame] + ")"
                    saved_frame_pos = self.m_comboBox_frame.FindString(saved_frame)
                    if saved_frame_pos != wx.NOT_FOUND:
                        self.m_comboBox_frame.SetSelection(saved_frame_pos)

            # Update the comboBox where user can select from which layer to take the popup menus
            if item in self.templates and "popups" in self.templates[item]:
                saved_popups = self.templates[item]["popups"]
            else:
                saved_popups = "Both Layers"
            if int(pcbnew.Version()[0:1]) < 8 and (saved_popups == "Front Layer" or saved_popups == "Back Layer"):
                saved_popups = "Both Layers"
            saved_popups_pos = self.m_comboBox_popups.FindString(saved_popups)
            if saved_popups_pos != wx.NOT_FOUND:
                self.m_comboBox_popups.SetSelection(saved_popups_pos)

            # Set the mirror checkbox according to saved setting
            if item in self.templates and "mirrored" in self.templates[item]:
                mirrored = self.templates[item]["mirrored"]
                self.m_checkBox_mirror.SetValue(bool(mirrored))

            # Set the tent checkbox according to saved setting
            if item in self.templates and "tented" in self.templates[item]:
                tented = self.templates[item]["tented"]
                self.m_checkBox_tent.SetValue(bool(tented))

    def OnLayerEdit(self, event):
        self.OnSaveLayer(self)
        selection = self.layersSortOrderBox.Selection
        if selection != wx.NOT_FOUND:
            self.disabledLayersSortOrderBox.SetSelection(-1)
            selected_item = self.layersSortOrderBox.GetString(selection)
            item = selected_item.split(' (')[0]  # Remove parenthesis if there is one
            self.current_layer = item
            self.show_layer_settings()

            color = self.layersColorDict.get(item, "#000000")
            self.m_textCtrl_color.ChangeValue(color)

            rgb_color = tuple(int(color.lstrip('#')[i:i + 2], 16) for i in (0, 2, 4))
            self.m_color_shower.SetBackgroundColour(rgb_color)
            self.m_color_shower.SetForegroundColour(rgb_color)
            self.m_color_shower.SetLabel(str(rgb_color))

            self.m_checkBox_negative.SetValue(self.layersNegativeDict.get(item, "false") == "true")
            self.m_checkBox_footprint_values.SetValue(self.layersFootprintValuesDict.get(item, "true") == "true")
            self.m_checkBox_reference_designators.SetValue(
                self.layersReferenceDesignatorsDict.get(item, "true") == "true")

    def OnTemplateNameChange(self, event):
        template_name = self.m_textCtrl_template_name.GetValue()
        item = re.sub('[^A-Za-z0-9\-\+ _]', '', template_name)
        if item != template_name:
            self.m_textCtrl_template_name.SetValue(item)
            return

        found_en = self.templatesSortOrderBox.FindString(item)
        found_dis = self.disabledTemplatesSortOrderBox.FindString(item)
        if found_en != wx.NOT_FOUND or found_dis != wx.NOT_FOUND:
            return

        self.SaveTemplate()

    def OnSaveLayer(self, event):
        if self.current_layer:
            def bool_str(value: bool):
                return "true" if value else "false"

            cl = self.current_layer
            self.layersColorDict[cl] = self.m_textCtrl_color.GetValue()
            self.layersNegativeDict[cl] = bool_str(self.m_checkBox_negative.IsChecked())
            self.layersFootprintValuesDict[cl] = self.m_checkBox_footprint_values.GetValue()
            self.layersFootprintValuesDict[cl] = bool_str(self.m_checkBox_footprint_values.IsChecked())
            self.layersReferenceDesignatorsDict[cl] = self.m_checkBox_reference_designators.GetValue()
            self.layersReferenceDesignatorsDict[cl] = bool_str(self.m_checkBox_reference_designators.IsChecked())
        self.SaveTemplate()

    # Helper functions
    def OnSize(self, event):
        # Trick the listCheckBox best size calculations
        tmp = self.templatesSortOrderBox.GetStrings()
        self.templatesSortOrderBox.SetItems([])
        self.Layout()
        self.templatesSortOrderBox.SetItems(tmp)

    def SaveTemplate(self):
        template_name = self.m_textCtrl_template_name.GetValue()
        if template_name:
            # Check if selected frame layer is enabled. Otherwise, add it to the bottom of the enabled list.
            frame_layer = self.m_comboBox_frame.GetValue()
            if frame_layer != "None":
                if self.layersSortOrderBox.FindString(frame_layer) == wx.NOT_FOUND:
                    self.layersSortOrderBox.Append(frame_layer)
                    frame_layer_pos = self.disabledLayersSortOrderBox.FindString(frame_layer)
                    if frame_layer_pos != wx.NOT_FOUND:
                        self.disabledLayersSortOrderBox.Delete(frame_layer_pos)

            enabled_layers_list = self.layersSortOrderBox.GetItems()
            # Remove the name within the parenthesis if there is one
            for i, layer_name in enumerate(enabled_layers_list):
                enabled_layers_list[i] = layer_name.split(' (')[0]

            enabled_layers = ','.join(enabled_layers_list)
            this_template = {"mirrored": self.m_checkBox_mirror.IsChecked(),
                             "tented": self.m_checkBox_tent.IsChecked(),
                             "enabled_layers": enabled_layers,
                             "frame": frame_layer.split(' (')[0],  # Remove parenthesis if there is one
                             "popups": self.m_comboBox_popups.GetValue(),
                             "layers": self.layersColorDict,
                             "layers_negative": self.layersNegativeDict,
                             "layers_footprint_values": self.layersFootprintValuesDict,
                             "layers_reference_designators": self.layersReferenceDesignatorsDict}
            if template_name != self.current_template:
                # Template has changed name. Remove the old name.
                self.templates.pop(self.current_template, None)
                # Change the name in the enabled templates list
                index = self.templatesSortOrderBox.FindString(self.current_template)
                if index != wx.NOT_FOUND:
                    self.templatesSortOrderBox.Delete(index)
                    self.templatesSortOrderBox.Insert(template_name, index)
                self.current_template = template_name
            self.templates[template_name] = this_template

    def ClearTemplateSettings(self):
        # Clear
        self.current_layer = ""
        self.m_textCtrl_template_name.ChangeValue("")
        self.m_checkBox_mirror.SetValue(False)
        self.m_checkBox_tent.SetValue(False)
        self.m_comboBox_frame.Clear()
        self.m_textCtrl_color.ChangeValue("")
        self.m_color_shower.SetBackgroundColour(wx.NullColour)
        self.m_color_shower.SetForegroundColour(wx.NullColour)
        self.m_color_shower.SetLabel("")
        self.m_checkBox_negative.SetValue(False)
        self.m_checkBox_footprint_values.SetValue(True)
        self.m_checkBox_reference_designators.SetValue(True)
        self.layersSortOrderBox.Clear()
        self.disabledLayersSortOrderBox.Clear()
        self.hide_layer_settings()



from configparser import ConfigParser
import json
import logging

_logger = logging.getLogger(__name__)


class Persistence:
    _options: dict = {
        # config (section, option): (varname, type conversion function)
        ('main', 'settings'): ('templates', json.loads),
        ('main', 'output_dest_dir'): ('output_path', None),
        ('main', 'enabled_templates'): ('enabled_templates', lambda s: [x for x in s.split(',') if x]),
        ('main', 'disabled_templates'): ('disabled_templates', lambda s: [x for x in s.split(',') if x]),
        ('main', 'create_svg'): ('create_svg', lambda x: x == "True"),
        ('main', 'del_temp_files'): ('del_temp_files', lambda x: x == "True"),
        ('main', 'delete_single_page_files'): ('del_single_page_files', lambda x: x == "True"),
        ('main', 'assembly_file_extension'): ('assembly_file_extension', None),
        ('main', 'layer_scale'): ('layer_scale', lambda x: float(x)),
    }
    _typeconv: dict = {
        bool: lambda x: "True" if x else "False",
        (list, tuple): lambda x: ",".join(x),
        dict: lambda x: json.dumps(x, indent=4),
        (int, float): lambda x: str(x),
        str: lambda x: x,
    }

    def __init__(self, configfile: str):
        self._config: ConfigParser = ConfigParser()
        self._configfile: str = configfile

        self.templates: dict = {}
        self.output_path: str = ''
        self.enabled_templates: list = []
        self.disabled_templates: list = []
        self.create_svg: bool = False
        self.del_temp_files: bool = False
        self.del_single_page_files: bool = False
        self.assembly_file_extension: str = "__Assembly"
        self.layer_scale: float = 1.0
        
        self.default_settings_file_path: str = ''
        self.global_settings_file_path: str = ''
        self.local_settings_file_path: str = ''

    def save(self, file_path: str):
        _logger.debug(f"save config")
        self._config.read(self._configfile)

        # `value` type conversion to string
        for (section, option), (varname, str2type) in self._options.items():
            value = getattr(self, varname)
            for _types, convert in self._typeconv.items():
                _logger.info(f"{section},{option}: {varname}")
                if isinstance(value, _types):
                    value = convert(value)
                    break
            else:
                raise Exception(f"unknown value type {type(value)}")

            if not self._config.has_section(section):
                self._config.add_section(section)
            self._config.set(section, option, value)

        with open(file_path, 'w') as f:
            self._config.write(f)

    def load(self) -> dict:
        self._config.read(self._configfile)

        values = {}  # alternative output
        for (section, option), (varname, post_action) in self._options.items():
            if self._config.has_option(section, option):
                val = self._config.get(section, option)
                values[varname] = post_action(val) if post_action else val
                setattr(self, varname, values[varname])
                _logger.info(f"{varname}={values[varname]}")

        return values



import os
import shutil
import sys
import pcbnew
import wx
import re
import traceback
import tempfile
import logging
from pathlib import Path

try:
    from .pypdf import PdfReader, PdfWriter, PageObject, Transformation, generic
except ImportError:
    from pypdf import PdfReader, PdfWriter, PageObject, Transformation, generic

# Try to import PyMuPDF.
has_pymupdf = True
try:
    import pymupdf  # This imports PyMuPDF

except:
    try:
        import fitz as pymupdf  # This imports PyMuPDF using old name

    except:
        try:
            import fitz_old as pymupdf # This imports PyMuPDF using temporary old name

        except:
            has_pymupdf = False

# after pip uninstall PyMuPDF the import still works, but not `open()`
# check if it's possible to call pymupdf.open()
if has_pymupdf:
    try:
        pymupdf.open()
    except:
        has_pymupdf = False


_logger = logging.getLogger(__name__)


def exception_msg(info: str, tb=True):
    msg = f"{info}\n\n" + (
        traceback.format_exc() if tb else '')
    try:
        wx.MessageBox(msg, 'Error', wx.OK | wx.ICON_ERROR)
    except wx._core.PyNoAppError:
        print(f'Error: {msg}', file=sys.stderr)


def io_file_error_msg(function: str, input_file: str, folder: str, more: str = '', tb=True):
    msg = f"{function} failed\nOn input file {input_file} in {folder}\n\n{more}" + (
        traceback.format_exc() if tb else '')
    try:
        wx.MessageBox(msg, 'Error', wx.OK | wx.ICON_ERROR)
    except wx._core.PyNoAppError:
        print(f'Error: {msg}', file=sys.stderr)


def colorize_pdf_pymupdf(folder, input_file, output_file, color):
    try:
        with pymupdf.open(os.path.join(folder, input_file)) as doc:
            xref_number = doc[0].get_contents()
            stream_bytes = doc.xref_stream(xref_number[0])
            new_color = ''.join([f'{c:.3g} ' for c in color])
            _logger.debug(f'{new_color=}')
            stream_bytes = re.sub(br'(\s)0 0 0 (RG|rg)', bytes(fr'\g<1>{new_color}\g<2>', 'ascii'), stream_bytes)

            doc.update_stream(xref_number[0], stream_bytes)
            doc.save(os.path.join(folder, output_file), clean=True)

    except RuntimeError as e:
        if "invalid key in dict" in str(e):
            io_file_error_msg(colorize_pdf_pymupdf.__name__, input_file, folder,
                              "This error can be due to PyMuPdf not being able to handle pdfs created by KiCad 7.0.1 due to a bug in KiCad 7.0.1. Upgrade KiCad or switch to pypdf instead.\n\n")
        return False

    except:
        io_file_error_msg(colorize_pdf_pymupdf.__name__, input_file, folder)
        return False

    return True


def colorize_pdf_pypdf(folder, input_file, output_file, color):
    try:
        with open(os.path.join(folder, input_file), "rb") as f:
            class ErrorHandler(object):
                def write(self, data):
                    if "UserWarning" not in data:
                        io_file_error_msg(colorize_pdf_pypdf.__name__, input_file, folder, data + "\n\n", tb=False)
                        return False

            if sys.stderr is None:
                handler = ErrorHandler()
                sys.stderr = handler

            source = PdfReader(f)
            output = PdfWriter()

            page = source.pages[0]
            content_object = page["/Contents"].get_object()
            content = generic.ContentStream(content_object, source)

            for i, (operands, operator) in enumerate(content.operations):
                if operator in (b"rg", b"RG"):
                    if operands == [0, 0, 0]:
                        content.operations[i] = (
                            [generic.FloatObject(intensity) for intensity in color], operator)
                    # else:
                    #    print(operator, operands[0], operands[1], operands[2], "The type is : ", type(operands[0]),
                    #          type(operands[1]), type(operands[2]))

            page[generic.NameObject("/Contents")] = content
            output.add_page(page)

            with open(os.path.join(folder, output_file), "wb") as output_stream:
                output.write(output_stream)

    except Exception:
        io_file_error_msg(colorize_pdf_pypdf.__name__, input_file, folder)
        return False

    return True

def merge_pdf_pymupdf(input_folder: str, input_files: list, output_folder: str, output_file: str, frame_file: str,
                    layer_scale: float, template_use_popups: bool, template_name: str):
    # I haven't found a way to scale the pdf and preserve the popup-menus.
    # For now, I'm taking the easy way out and handle the merging differently depending
    # on if scaling is used or not. At least the popup-menus are preserved when not using scaling.
    # https://github.com/pymupdf/PyMuPDF/discussions/2499
    # If popups aren't used, I'm using the with_scaling method to get rid of annotations
    if template_use_popups and layer_scale == 1.0:
        return merge_pdf_pymupdf_without_scaling(input_folder, input_files, output_folder, output_file, frame_file, template_name)
    else:
        return merge_pdf_pymupdf_with_scaling(input_folder, input_files, output_folder, output_file, frame_file, template_name, layer_scale)

def merge_pdf_pymupdf_without_scaling(input_folder: str, input_files: list, output_folder: str, output_file: str, frame_file: str, template_name: str):
    try:
        output = None
        for filename in reversed(input_files):
            try:
                if output is None:
                    output = pymupdf.open(os.path.join(input_folder, filename))
                else:
                    # using "with" to force RAII and avoid another "for" closing files
                    with pymupdf.open(os.path.join(input_folder, filename)) as src:
                        output[0].show_pdf_page(src[0].rect,  # select output rect
                                                src,  # input document
                                                overlay=False)
            except Exception:
                io_file_error_msg(merge_pdf_pymupdf.__name__, filename, input_folder)
                return False

        # Set correct page name in the table of contents (pdf outline)

        # toc = output.get_toc(simple=False)
        # print("Toc: ", toc)
        # toc[0][1] = template_name
        # output.set_toc(toc)
        # The above code doesn't work for the toc (outlines) of a pdf created by Kicad.
        # It correctly sets the name of the first page, but when clicking on a footprint it no longer zooms to that footprint

        # Lets do it the low-level way instead:
        try:
            xref = output.pdf_catalog()  # get xref of the /Catalog

            # print(output.xref_object(xref))  # print object definition
            # for key in output.xref_get_keys(xref): # iterate over all keys and print the keys and values
            #     print("%s = %s" % (key, output.xref_get_key(xref, key)))

            # The loop will output something like this:
            # Type = ('name', '/Catalog')
            # Pages = ('xref', '1 0 R')
            # Version = ('name', '/1.5')
            # PageMode = ('name', '/UseOutlines')
            # Outlines = ('xref', '20 0 R')
            # Names = ('xref', '4 0 R')
            # PageLayout = ('name', '/SinglePage')

            key_value = output.xref_get_key(xref, "Outlines") # Get the value of the 'Outlines' key
            xref = int(key_value[1].split(' ')[0]) # Set xref to the xref found in the value of the 'Outlines' key

            # The object now referenced by xref looks something like this:
            # Type = ('name', '/Outlines')
            # Count = ('int', '3')
            # First = ('xref', '11 0 R')
            # Last = ('xref', '11 0 R')

            key_value = output.xref_get_key(xref, "First") # Get the value of the 'First' key
            xref = int(key_value[1].split(' ')[0]) # Set xref to the xref found in the value of the 'First' key

            # The object now referenced by xref looks something like this:
            # Title = ('string', 'Page 1')
            # Parent = ('xref', '20 0 R')
            # Count = ('int', '-1')
            # First = ('xref', '12 0 R')
            # Last = ('xref', '12 0 R')
            # A = ('xref', '10 0 R')

            if output.xref_get_key(xref, "Title")[0] == 'string': # Check if the 'Title' key exists
                page_name = "(" + template_name + ")"
                output.xref_set_key(xref, "Title", page_name)

        except Exception:
            exception_msg("Didn't manage to set the name of the page in the Table-of-content")

        output.save(os.path.join(output_folder, output_file)) # , garbage=2
        output.close()

    except Exception:
        io_file_error_msg(merge_pdf_pymupdf.__name__, output_file, output_folder)
        return False

    return True

def merge_pdf_pymupdf_with_scaling(input_folder: str, input_files: list, output_folder: str, output_file: str, frame_file: str,
                    template_name: str, layer_scale: float):
    try:
        output = pymupdf.open()
        page = None
        for filename in reversed(input_files):
            try:
                # using "with" to force RAII and avoid another "for" closing files
                with pymupdf.open(os.path.join(input_folder, filename)) as src:
                    if page is None:
                        page = output.new_page(width=src[0].rect.width * layer_scale,
                                               height=src[0].rect.height * layer_scale)
                        cropbox = pymupdf.Rect((page.rect.width - src[0].rect.width) / 2,
                                            (page.rect.height - src[0].rect.height) / 2,
                                            (page.rect.width + src[0].rect.width) / 2,
                                            (page.rect.height + src[0].rect.height) / 2)

                    pos = cropbox if frame_file == filename else page.rect
                    page.show_pdf_page(pos,  # select output rect
                                       src,  # input document
                                       overlay=False)
            except Exception:
                io_file_error_msg(merge_pdf_pymupdf.__name__, filename, input_folder)
                return False

        page.set_cropbox(cropbox)

        # Set correct page name in the table of contents (pdf outline)
        # When scaling is used, component references will not be retained
        toc = [[1, template_name, 1]]
        output.set_toc(toc)

        output.save(os.path.join(output_folder, output_file))

    except Exception:
        io_file_error_msg(merge_pdf_pymupdf.__name__, output_file, output_folder)
        return False

    return True

def merge_pdf_pypdf(input_folder: str, input_files: list, output_folder: str, output_file: str, frame_file: str,
                    layer_scale: float, template_use_popups: bool, template_name: str):
    try:
        page = None
        for filename in input_files:
            try:
                filepath = os.path.join(input_folder, filename)
                pdf_reader = PdfReader(filepath)
                src_page: PageObject = pdf_reader.pages[0]

                op = Transformation()
                if layer_scale > 1.0:
                    if filename == frame_file:
                        x_offset = src_page.mediabox.width * (layer_scale - 1.0) / 2
                        y_offset = src_page.mediabox.height * (layer_scale - 1.0) / 2
                        op = op.translate(x_offset, y_offset)
                    else:
                        op = op.scale(layer_scale)

                if page is None:
                    page = PageObject.create_blank_page(width=src_page.mediabox.width * layer_scale,
                                                        height=src_page.mediabox.height * layer_scale)
                    page.cropbox.lower_left = ((page.mediabox.width - src_page.mediabox.width) / 2,
                                               (page.mediabox.height - src_page.mediabox.height) / 2)
                    page.cropbox.upper_right = ((page.mediabox.width + src_page.mediabox.width) / 2,
                                                (page.mediabox.height + src_page.mediabox.height) / 2)

                page.merge_transformed_page(src_page, op)

            except Exception:
                error_bitmap = ""
                error_msg = traceback.format_exc()
                if 'KeyError: 0' in error_msg:
                    error_bitmap = "This error can be caused by the presence of a bitmap image on this layer. Bitmaps are only allowed on the layer furthest down in the layer list. See Issue #11 for more information.\n\n"
                io_file_error_msg(merge_pdf_pypdf.__name__, filename, input_folder, error_bitmap)
                return False

        output = PdfWriter()
        output.add_page(page)
        output.add_outline_item(title=template_name, page_number=0)
        with open(os.path.join(output_folder, output_file), "wb") as output_stream:
            output.write(output_stream)

    except:
        io_file_error_msg(merge_pdf_pypdf.__name__, output_file, output_folder)
        return False

    return True


def create_pdf_from_pages(input_folder, input_files, output_folder, output_file, use_popups):
    try:
        output = PdfWriter()
        for filename in input_files:
            try:
                output.append(os.path.join(input_folder, filename))

            except:
                io_file_error_msg(create_pdf_from_pages.__name__, filename, input_folder)
                return False

        # If popup menus are used, add the needed javascript. Pypdf and PyMuPdf removes this in most operations.
        if use_popups:
            javascript_string = "function ShM(aEntries) { var aParams = []; for (var i in aEntries) { aParams.push({ cName: aEntries[i][0], cReturn: aEntries[i][1] }) } var cChoice = app.popUpMenuEx.apply(app, aParams); if (cChoice != null && cChoice.substring(0, 1) == '#') this.pageNum = parseInt(cChoice.slice(1)); else if (cChoice != null && cChoice.substring(0, 4) == 'http') app.launchURL(cChoice); }"
            output.add_js(javascript_string)

        for page in output.pages:
            # This has to be done on the writer, not the reader!
            page.compress_content_streams()  # This is CPU intensive!

        output.write(os.path.join(output_folder, output_file))

    except:
        io_file_error_msg(create_pdf_from_pages.__name__, output_file, output_folder)
        return False

    return True


class LayerInfo:
    std_color = "#000000"

    def __init__(self, layer_names: dict, layer_name: str, template: dict, frame_layer: str, popups: str):
        self.name: str = layer_name
        self.id: int = layer_names[layer_name]
        self.color_hex: str = template["layers"].get(layer_name, self.std_color)  # color as '#rrggbb' hex string
        self.with_frame: bool = layer_name == frame_layer

        try:
            # Bool specifying if layer is negative
            self.negative = template["layers_negative"][layer_name] == "true"
        except KeyError:
            self.negative = False

        try:
            # Bool specifying if footprint values shall be plotted
            self.footprint_value = template["layers_footprint_values"][layer_name] == "true"
        except KeyError:
            self.footprint_value = True

        try:
            # Bool specifying if footprint references shall be plotted
            self.reference_designator = template["layers_reference_designators"][layer_name] == "true"
        except KeyError:
            self.reference_designator = True

        # Check the popup settings.
        self.front_popups = True
        self.back_popups = True
        if popups == "Front Layer":
            self.back_popups = False
        elif popups == "Back Layer":
            self.front_popups = False
        elif popups == "None":
            self.front_popups = False
            self.back_popups = False

    @property
    def has_color(self) -> bool:
        """Checks if the layer color is not the standard color (=black)."""
        return self.color_hex != self.std_color

    @property
    def color_rgb(self) -> tuple[float, float, float]:
        """Return (red, green, blue) in float between 0-1."""
        value = self.color_hex.lstrip('#')
        lv = len(value)
        rgb = tuple(int(value[i:i + lv // 3], 16) for i in range(0, lv, lv // 3))
        rgb = (rgb[0] / 255, rgb[1] / 255, rgb[2] / 255)
        return rgb

    def __repr__(self):
        var_str = ', '.join(f"{key}: {value}" for key, value in vars(self).items())
        return f'{self.__class__.__name__}:{{ {var_str} }}'


class Template:
    def __init__(self, name: str, template: dict, layer_names: dict):
        self.name: str = name  # the template name
        self.mirrored: bool = template.get("mirrored", False)  # template is mirrored or not
        self.tented: bool = template.get("tented", False)  # template is tented or not

        frame_layer: str = template.get("frame", "")  # layer name of the frame layer
        popups: str = template.get("popups", "")  # setting saying if popups shall be taken from front, back or both

        # collection the settings of the enabled layers
        self.settings: list[LayerInfo] = []

        if "enabled_layers" in template:
            enabled_layers = template["enabled_layers"].split(',')
            for el in enabled_layers:
                if el:
                    # If this is the first layer, use the popup settings.
                    if el == enabled_layers[0]:
                        layer_popups: str  = popups
                    else:
                        layer_popups: str  = "None"
                    # Prepend to settings
                    layer_info = LayerInfo(layer_names, el, template, frame_layer, layer_popups)
                    self.settings.insert(0, layer_info)

    @property
    def steps(self) -> int:
        """number of process steps for this template"""
        return len(self.settings) + sum([layer.has_color for layer in self.settings])

    def __repr__(self):
        var_str = ', '.join(f"{key}: {value}" for key, value in vars(self).items())
        return f'{self.__class__.__name__}:{{ {var_str} }}'


def plot_pdfs(board, output_path, templates, enabled_templates, del_temp_files, create_svg, del_single_page_files,
                 dlg=None, **kwargs) -> bool:
    asy_file_extension = kwargs.pop('assembly_file_extension', '__Assembly')
    layer_scale = kwargs.pop('layer_scale', 1.0)
    colorize_lib: str = kwargs.pop('colorize_lib', '')
    merge_lib: str = kwargs.pop('merge_lib', '')

    if dlg is None:
        use_pymupdf = has_pymupdf or create_svg
        pymupdf_pdf = has_pymupdf and colorize_lib != 'pypdf'
        pymupdf_merge = has_pymupdf and merge_lib != 'pypdf'

        def set_progress_status(progress: int, status: str):
            print(f'{int(progress):3d}%: {status}')

        def msg_box(text, caption, flags):
            print(f"{caption}: {text}")

    elif isinstance(dlg, wx.Panel):
        pymupdf_pdf = dlg.m_radio_pymupdf.GetValue()
        pymupdf_merge = dlg.m_radio_merge_pymupdf.GetValue()
        use_pymupdf = pymupdf_pdf or pymupdf_merge or create_svg

        def set_progress_status(progress: int, status: str):
            dlg.m_staticText_status.SetLabel(f'Status: {status}')
            dlg.m_progress.SetValue(int(progress))
            dlg.Refresh()
            dlg.Update()

        def msg_box(text, caption, flags):
            wx.MessageBox(text, caption, flags)
    else:
        print(f"Error: Unknown dialog type {type(dlg)}", file=sys.stderr)
        return False

    if use_pymupdf and not has_pymupdf:
        msg_box(
            "PyMuPdf wasn't loaded.\n\nIt must be installed for it to be used for coloring, for merging and for creating SVGs.\n\nMore information under Install dependencies in the Wiki at board2pdf.dennevi.com",
            'Error', wx.OK | wx.ICON_ERROR)
        set_progress_status(100, "Failed to load PyMuPDF.")
        return False

    colorize_pdf = colorize_pdf_pymupdf if pymupdf_pdf else colorize_pdf_pypdf
    merge_pdf = merge_pdf_pymupdf if pymupdf_merge else merge_pdf_pypdf

    os.chdir(os.path.dirname(board.GetFileName()))
    output_dir = os.path.abspath(os.path.expanduser(os.path.expandvars(output_path)))
    if del_temp_files:
        # in case the files are deleted: use the OS temp directory
        temp_dir = tempfile.mkdtemp()
    else:
        temp_dir = os.path.abspath(os.path.join(output_dir, "temp"))

    progress = 5
    set_progress_status(progress, "Started plotting...")

    plot_controller = pcbnew.PLOT_CONTROLLER(board)
    plot_options = plot_controller.GetPlotOptions()

    base_filename = os.path.basename(os.path.splitext(board.GetFileName())[0])
    final_assembly_file = base_filename + asy_file_extension + ".pdf"
    final_assembly_file_with_path = os.path.abspath(os.path.join(output_dir, final_assembly_file))

    if "assembly_file_output" in kwargs:
        final_assembly_file = kwargs.pop('assembly_file_output')
        final_assembly_file_with_path = str(Path(final_assembly_file_with_path).absolute())


    # Create the directory if it doesn't exist already
    os.makedirs(output_dir, exist_ok=True)

    # Check if we're able to write to the output file.
    try:
        # os.access(os.path.join(output_dir, final_assembly_file), os.W_OK)
        open(os.path.join(output_dir, final_assembly_file), "w")
    except:
        msg_box("The output file is not writeable. Perhaps it's open in another application?\n\n"
                + final_assembly_file_with_path, 'Error', wx.OK | wx.ICON_ERROR)
        set_progress_status(100, "Failed to write to output file.")
        return False

    plot_options.SetOutputDirectory(temp_dir)

    # Build a dict to translate layer names to layerID
    layer_names = {}
    for i in range(pcbnew.PCBNEW_LAYER_ID_START, pcbnew.PCBNEW_LAYER_ID_START + pcbnew.PCB_LAYER_ID_COUNT):
        layer_names[board.GetStandardLayerName(i)] = i

    steps: int = 2  # number of process steps
    templates_list: list[Template] = []
    for t in enabled_templates:
        # {  "Test-template": {"mirrored": true, "enabled_layers": "B.Fab,B.Mask,Edge.Cuts,F.Adhesive", "frame": "In4.Cu",
        #          "layers": {"B.Fab": "#000012", "B.Mask": "#000045"}}  }
        if t in templates:
            temp = Template(t, templates[t], layer_names)
            _logger.debug(temp)
            steps += 1 + temp.steps
            templates_list.append(temp)
    progress_step: float = 95 / steps

    """
    [
        ["Greyscale Top", False,
            [
             ("F_Cu", pcbnew.F_Cu, "#F0F0F0", False, True, True, True),
             ("F_Paste", pcbnew.F_Paste, "#C4C4C4", False, False, True, True),
            ]
        ],
    ]
    """
    try:
        # Set General Options:
        plot_options.SetPlotInvisibleText(False)
        # plot_options.SetPlotPadsOnSilkLayer(False);
        plot_options.SetUseAuxOrigin(False)
        plot_options.SetScale(1.0)
        plot_options.SetAutoScale(False)
        # plot_options.SetPlotMode(PLOT_MODE)
        # plot_options.SetLineWidth(2000)
        if pcbnew.Version()[0:3] == "6.0":
            # This method is only available on V6, not V6.99/V7
            plot_options.SetExcludeEdgeLayer(True)
    except:
        msg_box(traceback.format_exc(), 'Error', wx.OK | wx.ICON_ERROR)
        set_progress_status(100, "Failed to set plot_options")
        return False

    use_popups = False
    template_filelist = []

    # Iterate over the templates
    for template in templates_list:
        # msg_box("Now starting with template: " + template_name)
        # Plot layers to pdf files
        for layer_info in template.settings:
            progress += progress_step
            set_progress_status(progress, f"Plotting {layer_info.name} for template {template.name}")

            if pcbnew.Version()[0:3] == "6.0":
                if pcbnew.IsCopperLayer(layer_info.id):  # Should probably do this on mask layers as well
                    plot_options.SetDrillMarksType(
                        2)  # NO_DRILL_SHAPE = 0, SMALL_DRILL_SHAPE = 1, FULL_DRILL_SHAPE  = 2
                else:
                    plot_options.SetDrillMarksType(
                        0)  # NO_DRILL_SHAPE = 0, SMALL_DRILL_SHAPE = 1, FULL_DRILL_SHAPE  = 2
            else:  # API changed in V6.99/V7
                try:
                    if pcbnew.IsCopperLayer(layer_info.id):  # Should probably do this on mask layers as well
                        plot_options.SetDrillMarksType(pcbnew.DRILL_MARKS_FULL_DRILL_SHAPE)
                    else:
                        plot_options.SetDrillMarksType(pcbnew.DRILL_MARKS_NO_DRILL_SHAPE)
                except:
                    msg_box(
                        "Unable to set Drill Marks type.\n\nIf you're using a V6.99 build from before Dec 07 2022 then update to a newer build.\n\n" + traceback.format_exc(),
                        'Error', wx.OK | wx.ICON_ERROR)
                    set_progress_status(100, "Failed to set Drill Marks type")

                    return False

            try:
                plot_options.SetPlotFrameRef(layer_info.with_frame)
                plot_options.SetNegative(layer_info.negative)
                plot_options.SetPlotValue(layer_info.footprint_value)
                plot_options.SetPlotReference(layer_info.reference_designator)
                plot_options.SetMirror(template.mirrored)
                plot_options.SetPlotViaOnMaskLayer(template.tented)
                if int(pcbnew.Version()[0:1]) >= 8:
                    if layer_scale == 1.0:
                        plot_options.m_PDFFrontFPPropertyPopups = layer_info.front_popups
                        plot_options.m_PDFBackFPPropertyPopups = layer_info.back_popups
                    else:
                        plot_options.m_PDFFrontFPPropertyPopups = False
                        plot_options.m_PDFBackFPPropertyPopups = False

                plot_controller.SetLayer(layer_info.id)
                plot_controller.OpenPlotfile(layer_info.name, pcbnew.PLOT_FORMAT_PDF, template.name)
                plot_controller.PlotLayer()
            except:
                msg_box(traceback.format_exc(), 'Error', wx.OK | wx.ICON_ERROR)
                set_progress_status(100, "Failed to set plot_options or plot_controller")
                return False

        plot_controller.ClosePlot()

        template_use_popups = False
        filelist = []
        # Change color of pdf files
        for layer_info in template.settings:
            ln = layer_info.name.replace('.', '_')
            input_file = f"{base_filename}-{ln}.pdf"
            output_file = f"{base_filename}-{ln}-colored.pdf"
            if layer_info.has_color:
                progress += progress_step
                set_progress_status(progress, f"Coloring {layer_info.name} for template {template.name}")

                if not colorize_pdf(temp_dir, input_file, output_file, layer_info.color_rgb):
                    set_progress_status(100, f"Failed when coloring {layer_info.name} for template {template.name}")
                    return False

                filelist.append(output_file)
            else:
                filelist.append(input_file)

            if layer_info.with_frame:
                # the frame layer is scaled by 1.0, all others by `layer_scale`
                frame_file = filelist[-1]
            else:
                frame_file = 'None'
            # Set template_use_popups to True if any layer has popups and no scaling
            if layer_scale == 1.0:
                template_use_popups = template_use_popups or layer_info.front_popups or layer_info.back_popups

        # Merge pdf files
        progress += progress_step
        set_progress_status(progress, f"Merging all layers of template {template.name}")

        assembly_file = f"{base_filename}_{template.name}.pdf"

        if not merge_pdf(temp_dir, filelist, output_dir, assembly_file, frame_file, layer_scale, template_use_popups, template.name):
            set_progress_status(100, "Failed when merging all layers of template " + template.name)
            return False

        template_filelist.append(assembly_file)
        # Set use_popups to True if any template has popups
        use_popups = use_popups or template_use_popups

    # Add all generated pdfs to one file
    progress += progress_step
    set_progress_status(progress, "Adding all templates to a single file")


    if not create_pdf_from_pages(output_dir, template_filelist, output_dir, final_assembly_file, use_popups):
        set_progress_status(100, "Failed when adding all templates to a single file")
        return False

    # Create SVG(s) if settings says so
    if create_svg:
        for template_file in template_filelist:
            template_pdf = pymupdf.open(os.path.join(output_dir, template_file))
            try:
                svg_image = template_pdf[0].get_svg_image()
                svg_filename = os.path.splitext(template_file)[0] + ".svg"
                with open(os.path.join(output_dir, svg_filename), "w") as file:
                    file.write(svg_image)
            except:
                msg_box("Failed to create SVG in {output_dir}\n\n" + traceback.format_exc(), 'Error',
                        wx.OK | wx.ICON_ERROR)
                set_progress_status(100, "Failed to create SVG(s)")
                return False
            template_pdf.close()

    # Delete temp files if setting says so
    if del_temp_files:
        try:
            shutil.rmtree(temp_dir)
        except:
            msg_box(f"del_temp_files failed\n\nOn dir {temp_dir}\n\n" + traceback.format_exc(), 'Error',
                    wx.OK | wx.ICON_ERROR)
            set_progress_status(100, "Failed to delete temp files")
            return False

    # Delete single page files if setting says so
    if del_single_page_files:
        for template_file in template_filelist:
            delete_file = os.path.join(output_dir, os.path.splitext(template_file)[0] + ".pdf")
            try:
                os.remove(delete_file)
            except:
                msg_box(f"del_single_page_files failed\n\nOn file {delete_file}\n\n" + traceback.format_exc(), 'Error',
                        wx.OK | wx.ICON_ERROR)
                set_progress_status(100, "Failed to delete single files")
                return False

    set_progress_status(100, "All done!")

    endmsg = "Assembly pdf created: " + os.path.abspath(os.path.join(output_dir, final_assembly_file))
    if not del_single_page_files:
        endmsg += "\n\nSingle page pdf files created:"
        for template_file in template_filelist:
            endmsg += "\n" + os.path.abspath(os.path.join(output_dir, os.path.splitext(template_file)[0] + ".pdf"))

    if create_svg:
        endmsg += "\n\nSVG files created:"
        for template_file in template_filelist:
            endmsg += "\n" + os.path.abspath(os.path.join(output_dir, os.path.splitext(template_file)[0] + ".svg"))

    msg_box(endmsg, 'All done!', wx.OK)
    return True


def cli(board_filepath: str, configfile: str, **kwargs) -> bool:
    try:
        from . import persistence
    except ImportError:
        import persistence


    board = pcbnew.LoadBoard(board_filepath)
    config = persistence.Persistence(configfile)
    config_vars = config.load()
    # note: cli parameters override config.ini values
    config_vars.update(kwargs)
    return plot_pdfs(board, **config_vars)



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 4.1.0-0-g733bf3d)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class SettingsDialogBase
###########################################################################

class SettingsDialogBase ( wx.Dialog ):

    def __init__( self, parent ):
        wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Board2Pdf", pos = wx.DefaultPosition, size = wx.Size( 900,670 ), style = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER|wx.BORDER_DEFAULT )

        self.SetSizeHints( wx.Size( 900,670 ), wx.DefaultSize )


        self.Centre( wx.BOTH )

    def __del__( self ):
        pass


###########################################################################
## Class SettingsDialogPanel
###########################################################################

class SettingsDialogPanel ( wx.Panel ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 900,760 ), style = wx.TAB_TRAVERSAL, name = wx.EmptyString ):
        wx.Panel.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        bSizer20 = wx.BoxSizer( wx.HORIZONTAL )

        bSizer30 = wx.BoxSizer( wx.VERTICAL )

        sbSizer22 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"General Settings" ), wx.VERTICAL )

        bSizer32 = wx.BoxSizer( wx.VERTICAL )

        sortingSizer = wx.StaticBoxSizer( wx.StaticBox( sbSizer22.GetStaticBox(), wx.ID_ANY, u"Enabled Templates" ), wx.VERTICAL )

        bSizer4 = wx.BoxSizer( wx.HORIZONTAL )

        bSizer6 = wx.BoxSizer( wx.VERTICAL )

        templatesSortOrderBoxChoices = []
        self.templatesSortOrderBox = wx.ListBox( sortingSizer.GetStaticBox(), wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, templatesSortOrderBoxChoices, wx.LB_SINGLE|wx.BORDER_SIMPLE )
        self.templatesSortOrderBox.SetToolTip( u"Double-click to disable template" )

        bSizer6.Add( self.templatesSortOrderBox, 1, wx.ALL|wx.EXPAND, 5 )


        bSizer4.Add( bSizer6, 1, wx.EXPAND, 5 )

        bSizer5 = wx.BoxSizer( wx.VERTICAL )

        self.m_button_template_up = wx.Button( sortingSizer.GetStaticBox(), wx.ID_ANY, u"Up", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button_template_up.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_GO_UP, wx.ART_BUTTON ) )
        self.m_button_template_up.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer5.Add( self.m_button_template_up, 0, wx.ALL, 5 )

        self.m_button_template_down = wx.Button( sortingSizer.GetStaticBox(), wx.ID_ANY, u"Down", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button_template_down.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_GO_DOWN, wx.ART_BUTTON ) )
        self.m_button_template_down.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer5.Add( self.m_button_template_down, 0, wx.ALL, 5 )

        self.m_button_template_new = wx.Button( sortingSizer.GetStaticBox(), wx.ID_ANY, u"New", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button_template_new.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_NEW, wx.ART_BUTTON ) )
        self.m_button_template_new.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer5.Add( self.m_button_template_new, 0, wx.ALL, 5 )

        self.m_button_template_copy = wx.Button( sortingSizer.GetStaticBox(), wx.ID_ANY, u"Clone", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button_template_copy.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_COPY, wx.ART_BUTTON ) )
        self.m_button_template_copy.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer5.Add( self.m_button_template_copy, 0, wx.ALL, 5 )

        self.m_button_template_disable = wx.Button( sortingSizer.GetStaticBox(), wx.ID_ANY, u"Disable", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button_template_disable.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_MINUS, wx.ART_BUTTON ) )
        self.m_button_template_disable.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer5.Add( self.m_button_template_disable, 0, wx.ALL, 5 )


        bSizer4.Add( bSizer5, 0, wx.ALIGN_RIGHT, 5 )


        sortingSizer.Add( bSizer4, 1, wx.EXPAND, 5 )

        self.m_staticText81 = wx.StaticText( sortingSizer.GetStaticBox(), wx.ID_ANY, u"Note! Template at the top of the list will become page one in pdf, and so on.", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText81.Wrap( -1 )

        sortingSizer.Add( self.m_staticText81, 0, wx.ALL, 5 )


        bSizer32.Add( sortingSizer, 1, wx.ALL|wx.EXPAND, 5 )

        layersSizer = wx.StaticBoxSizer( wx.StaticBox( sbSizer22.GetStaticBox(), wx.ID_ANY, u"Disabled Templates" ), wx.VERTICAL )

        bSizer41 = wx.BoxSizer( wx.HORIZONTAL )

        bSizer61 = wx.BoxSizer( wx.VERTICAL )

        disabledTemplatesSortOrderBoxChoices = []
        self.disabledTemplatesSortOrderBox = wx.ListBox( layersSizer.GetStaticBox(), wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, disabledTemplatesSortOrderBoxChoices, wx.LB_SINGLE|wx.LB_SORT|wx.BORDER_SIMPLE )
        self.disabledTemplatesSortOrderBox.SetToolTip( u"Double-click to enable template" )

        bSizer61.Add( self.disabledTemplatesSortOrderBox, 1, wx.ALL|wx.EXPAND, 5 )


        bSizer41.Add( bSizer61, 1, wx.EXPAND, 5 )

        bSizer51 = wx.BoxSizer( wx.VERTICAL )

        self.m_button_template_enable = wx.Button( layersSizer.GetStaticBox(), wx.ID_ANY, u"Enable", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button_template_enable.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_PLUS, wx.ART_BUTTON ) )
        self.m_button_template_enable.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer51.Add( self.m_button_template_enable, 0, wx.ALL, 5 )

        self.m_button4 = wx.Button( layersSizer.GetStaticBox(), wx.ID_ANY, u"Del", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button4.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_DELETE, wx.ART_BUTTON ) )
        self.m_button4.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer51.Add( self.m_button4, 0, wx.ALL, 5 )


        bSizer41.Add( bSizer51, 0, wx.ALIGN_RIGHT, 5 )


        layersSizer.Add( bSizer41, 1, wx.EXPAND, 5 )


        bSizer32.Add( layersSizer, 1, wx.ALL|wx.EXPAND, 5 )

        sbSizer6 = wx.StaticBoxSizer( wx.StaticBox( sbSizer22.GetStaticBox(), wx.ID_ANY, u"Settings" ), wx.VERTICAL )

        fgSizer1 = wx.FlexGridSizer( 0, 2, 0, 0 )
        fgSizer1.AddGrowableCol( 1 )
        fgSizer1.SetFlexibleDirection( wx.BOTH )
        fgSizer1.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.m_staticText8 = wx.StaticText( sbSizer6.GetStaticBox(), wx.ID_ANY, u"Output Directory", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText8.Wrap( -1 )

        fgSizer1.Add( self.m_staticText8, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.outputDirPicker = wx.DirPickerCtrl( sbSizer6.GetStaticBox(), wx.ID_ANY, wx.EmptyString, u"Select bom folder", wx.DefaultPosition, wx.Size( -1,-1 ), wx.DIRP_USE_TEXTCTRL|wx.BORDER_SIMPLE )
        fgSizer1.Add( self.outputDirPicker, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL|wx.EXPAND, 5 )


        sbSizer6.Add( fgSizer1, 1, wx.EXPAND, 5 )

        bSizer201 = wx.BoxSizer( wx.HORIZONTAL )

        self.m_checkBox_create_svg = wx.CheckBox( sbSizer6.GetStaticBox(), wx.ID_ANY, u"Create SVGs", wx.DefaultPosition, wx.DefaultSize, 0 )
        bSizer201.Add( self.m_checkBox_create_svg, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.m_checkBox_delete_single_page_files = wx.CheckBox( sbSizer6.GetStaticBox(), wx.ID_ANY, u"Delete single page files", wx.DefaultPosition, wx.DefaultSize, 0 )
        bSizer201.Add( self.m_checkBox_delete_single_page_files, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.m_checkBox_delete_temp_files = wx.CheckBox( sbSizer6.GetStaticBox(), wx.ID_ANY, u"Delete temporary files", wx.DefaultPosition, wx.DefaultSize, 0 )
        bSizer201.Add( self.m_checkBox_delete_temp_files, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        sbSizer6.Add( bSizer201, 1, wx.ALIGN_RIGHT, 5 )

        bSizer21 = wx.BoxSizer( wx.HORIZONTAL )

        self.m_staticText82 = wx.StaticText( sbSizer6.GetStaticBox(), wx.ID_ANY, u"Library for coloring:", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText82.Wrap( -1 )

        bSizer21.Add( self.m_staticText82, 0, wx.ALL, 5 )

        self.m_radio_pymupdf = wx.RadioButton( sbSizer6.GetStaticBox(), wx.ID_ANY, u"PyMuPdf (fast)", wx.DefaultPosition, wx.DefaultSize, wx.RB_GROUP )
        bSizer21.Add( self.m_radio_pymupdf, 0, wx.ALL, 5 )

        self.m_radio_pypdf = wx.RadioButton( sbSizer6.GetStaticBox(), wx.ID_ANY, u"pypdf (slow)", wx.DefaultPosition, wx.DefaultSize, 0 )
        bSizer21.Add( self.m_radio_pypdf, 0, wx.ALL, 5 )


        sbSizer6.Add( bSizer21, 1, wx.EXPAND, 5 )

        bSizer22 = wx.BoxSizer( wx.HORIZONTAL )

        self.m_staticText9 = wx.StaticText( sbSizer6.GetStaticBox(), wx.ID_ANY, u"Library for merging:", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText9.Wrap( -1 )

        bSizer22.Add( self.m_staticText9, 0, wx.ALL, 5 )

        self.m_radio_merge_pymupdf = wx.RadioButton( sbSizer6.GetStaticBox(), wx.ID_ANY, u"PyMuPdf (fast)", wx.DefaultPosition, wx.DefaultSize, wx.RB_GROUP )
        bSizer22.Add( self.m_radio_merge_pymupdf, 0, wx.ALL, 5 )

        self.m_radio_merge_pypdf = wx.RadioButton( sbSizer6.GetStaticBox(), wx.ID_ANY, u"pypdf (slow)", wx.DefaultPosition, wx.DefaultSize, 0 )
        bSizer22.Add( self.m_radio_merge_pypdf, 0, wx.ALL, 5 )


        sbSizer6.Add( bSizer22, 1, wx.EXPAND, 5 )

        bSizer39 = wx.BoxSizer( wx.HORIZONTAL )

        self.saveSettingsBtn = wx.Button( sbSizer6.GetStaticBox(), wx.ID_ANY, u"Save settings", wx.DefaultPosition, wx.DefaultSize, 0|wx.BORDER_DEFAULT )

        self.saveSettingsBtn.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_FILE_SAVE, wx.ART_BUTTON ) )
        self.saveSettingsBtn.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer39.Add( self.saveSettingsBtn, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.loadSettingsBtn = wx.Button( sbSizer6.GetStaticBox(), wx.ID_ANY, u"Load settings", wx.DefaultPosition, wx.DefaultSize, 0|wx.BORDER_DEFAULT )

        self.loadSettingsBtn.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_FILE_OPEN, wx.ART_BUTTON ) )
        bSizer39.Add( self.loadSettingsBtn, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        bSizer39.Add( ( 0, 0), 1, wx.EXPAND, 5 )

        self.m_button42 = wx.Button( sbSizer6.GetStaticBox(), wx.ID_ANY, u"Perform", wx.DefaultPosition, wx.DefaultSize, 0|wx.BORDER_DEFAULT|wx.BORDER_RAISED )

        self.m_button42.SetDefault()

        self.m_button42.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_PRINT, wx.ART_CMN_DIALOG ) )
        self.m_button42.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer39.Add( self.m_button42, 0, wx.ALL, 5 )

        self.m_button43 = wx.Button( sbSizer6.GetStaticBox(), wx.ID_CANCEL, u"Exit", wx.DefaultPosition, wx.DefaultSize, 0|wx.BORDER_DEFAULT )

        self.m_button43.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_QUIT, wx.ART_CMN_DIALOG ) )
        self.m_button43.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer39.Add( self.m_button43, 0, wx.ALL, 5 )


        sbSizer6.Add( bSizer39, 0, wx.EXPAND, 5 )

        self.m_progress = wx.Gauge( sbSizer6.GetStaticBox(), wx.ID_ANY, 100, wx.DefaultPosition, wx.Size( 400,-1 ), wx.GA_HORIZONTAL )
        self.m_progress.SetValue( 0 )
        sbSizer6.Add( self.m_progress, 0, wx.ALL|wx.EXPAND, 5 )

        self.m_staticText_status = wx.StaticText( sbSizer6.GetStaticBox(), wx.ID_ANY, u"Status:", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText_status.Wrap( -1 )

        sbSizer6.Add( self.m_staticText_status, 0, wx.ALL, 5 )


        bSizer32.Add( sbSizer6, 0, wx.ALL|wx.EXPAND, 5 )


        sbSizer22.Add( bSizer32, 1, wx.EXPAND, 5 )


        bSizer30.Add( sbSizer22, 1, wx.EXPAND, 5 )


        bSizer20.Add( bSizer30, 1, wx.EXPAND, 5 )

        bSizer301 = wx.BoxSizer( wx.VERTICAL )

        sbSizer23 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Template Settings" ), wx.VERTICAL )

        m_sizer_template = wx.BoxSizer( wx.VERTICAL )

        sbSizer61 = wx.StaticBoxSizer( wx.StaticBox( sbSizer23.GetStaticBox(), wx.ID_ANY, u"Template Properties" ), wx.VERTICAL )

        gSizer1 = wx.GridSizer( 4, 2, 0, 0 )

        self.m_staticText_template_name = wx.StaticText( sbSizer61.GetStaticBox(), wx.ID_ANY, u"Template name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText_template_name.Wrap( -1 )

        gSizer1.Add( self.m_staticText_template_name, 0, wx.ALL, 5 )

        self.m_textCtrl_template_name = wx.TextCtrl( sbSizer61.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.Size( 190,-1 ), 0 )
        gSizer1.Add( self.m_textCtrl_template_name, 0, wx.ALL|wx.EXPAND, 5 )

        self.m_staticText_frame_layer = wx.StaticText( sbSizer61.GetStaticBox(), wx.ID_ANY, u"Draw frame on layer", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText_frame_layer.Wrap( -1 )

        gSizer1.Add( self.m_staticText_frame_layer, 0, wx.ALL, 5 )

        m_comboBox_frameChoices = []
        self.m_comboBox_frame = wx.ComboBox( sbSizer61.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.Size( 190,-1 ), m_comboBox_frameChoices, 0 )
        gSizer1.Add( self.m_comboBox_frame, 0, wx.ALL|wx.EXPAND, 5 )

        self.m_staticText_popups = wx.StaticText( sbSizer61.GetStaticBox(), wx.ID_ANY, u"Property Popups", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText_popups.Wrap( -1 )

        gSizer1.Add( self.m_staticText_popups, 0, wx.ALL, 5 )

        m_comboBox_popupsChoices = [ u"None", u"Front Layer", u"Back Layer", u"Both Layers", wx.EmptyString ]
        self.m_comboBox_popups = wx.ComboBox( sbSizer61.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.Size( 190,-1 ), m_comboBox_popupsChoices, 0 )
        gSizer1.Add( self.m_comboBox_popups, 0, wx.ALL|wx.EXPAND, 5 )

        self.m_checkBox_mirror = wx.CheckBox( sbSizer61.GetStaticBox(), wx.ID_ANY, u"Mirror all layers (bottom view)", wx.DefaultPosition, wx.DefaultSize, 0 )
        gSizer1.Add( self.m_checkBox_mirror, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.m_checkBox_tent = wx.CheckBox( sbSizer61.GetStaticBox(), wx.ID_ANY, u"Do not tent vias on solder mask", wx.DefaultPosition, wx.DefaultSize, 0 )
        gSizer1.Add( self.m_checkBox_tent, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        sbSizer61.Add( gSizer1, 1, wx.EXPAND, 5 )


        m_sizer_template.Add( sbSizer61, 0, wx.ALL|wx.EXPAND, 5 )

        sbSizer611 = wx.StaticBoxSizer( wx.StaticBox( sbSizer23.GetStaticBox(), wx.ID_ANY, u"Layer Properties" ), wx.VERTICAL )

        gSizer2 = wx.GridSizer( 0, 4, 0, 0 )

        self.m_staticText_layer_color = wx.StaticText( sbSizer611.GetStaticBox(), wx.ID_ANY, u"Layer color", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText_layer_color.Wrap( -1 )

        gSizer2.Add( self.m_staticText_layer_color, 0, wx.ALL, 5 )

        self.m_textCtrl_color = wx.TextCtrl( sbSizer611.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
        gSizer2.Add( self.m_textCtrl_color, 0, wx.ALL|wx.EXPAND, 5 )

        self.m_color_shower = wx.StaticText( sbSizer611.GetStaticBox(), wx.ID_ANY, u"test", wx.DefaultPosition, wx.Size( 80,20 ), 0 )
        self.m_color_shower.Wrap( -1 )

        self.m_color_shower.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )

        gSizer2.Add( self.m_color_shower, 0, wx.ALL, 5 )

        self.m_button_pick_color = wx.Button( sbSizer611.GetStaticBox(), wx.ID_ANY, u"Pick Color", wx.DefaultPosition, wx.Size( 70,-1 ), 0 )

        self.m_button_pick_color.SetBitmapMargins( wx.Size( 2,-1 ) )
        gSizer2.Add( self.m_button_pick_color, 0, wx.ALIGN_RIGHT|wx.ALL, 5 )


        sbSizer611.Add( gSizer2, 1, wx.EXPAND, 5 )

        gSizer3 = wx.GridSizer( 0, 3, 0, 0 )

        self.m_checkBox_negative = wx.CheckBox( sbSizer611.GetStaticBox(), wx.ID_ANY, u"Negative plot", wx.DefaultPosition, wx.DefaultSize, 0 )
        gSizer3.Add( self.m_checkBox_negative, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.m_checkBox_reference_designators = wx.CheckBox( sbSizer611.GetStaticBox(), wx.ID_ANY, u"Plot ref. designators", wx.DefaultPosition, wx.DefaultSize, 0 )
        gSizer3.Add( self.m_checkBox_reference_designators, 0, wx.ALL, 5 )

        self.m_checkBox_footprint_values = wx.CheckBox( sbSizer611.GetStaticBox(), wx.ID_ANY, u"Plot footprint values", wx.DefaultPosition, wx.DefaultSize, 0 )
        gSizer3.Add( self.m_checkBox_footprint_values, 0, wx.ALL, 5 )


        sbSizer611.Add( gSizer3, 1, wx.EXPAND, 5 )


        m_sizer_template.Add( sbSizer611, 0, wx.ALL|wx.EXPAND, 5 )

        sortingSizer1 = wx.StaticBoxSizer( wx.StaticBox( sbSizer23.GetStaticBox(), wx.ID_ANY, u"Enabled Layers" ), wx.VERTICAL )

        bSizer512 = wx.BoxSizer( wx.VERTICAL )

        bSizer42 = wx.BoxSizer( wx.HORIZONTAL )

        bSizer62 = wx.BoxSizer( wx.VERTICAL )

        layersSortOrderBoxChoices = []
        self.layersSortOrderBox = wx.ListBox( sortingSizer1.GetStaticBox(), wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, layersSortOrderBoxChoices, wx.LB_SINGLE|wx.BORDER_SIMPLE )
        self.layersSortOrderBox.SetToolTip( u"Double-click to disable layer" )

        bSizer62.Add( self.layersSortOrderBox, 1, wx.ALL|wx.EXPAND, 5 )


        bSizer42.Add( bSizer62, 1, wx.EXPAND, 5 )

        bSizer52 = wx.BoxSizer( wx.VERTICAL )

        self.m_button_layer_up = wx.Button( sortingSizer1.GetStaticBox(), wx.ID_ANY, u"Up", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button_layer_up.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_GO_UP, wx.ART_BUTTON ) )
        self.m_button_layer_up.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer52.Add( self.m_button_layer_up, 0, wx.ALL, 5 )

        self.m_button_layer_down = wx.Button( sortingSizer1.GetStaticBox(), wx.ID_ANY, u"Down", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button_layer_down.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_GO_DOWN, wx.ART_BUTTON ) )
        self.m_button_layer_down.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer52.Add( self.m_button_layer_down, 0, wx.ALL, 5 )

        self.m_button_layer_disable = wx.Button( sortingSizer1.GetStaticBox(), wx.ID_ANY, u"Disable", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button_layer_disable.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_MINUS, wx.ART_BUTTON ) )
        self.m_button_layer_disable.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer52.Add( self.m_button_layer_disable, 0, wx.ALL, 5 )


        bSizer42.Add( bSizer52, 0, wx.ALIGN_RIGHT, 5 )


        bSizer512.Add( bSizer42, 1, wx.EXPAND, 5 )

        self.m_staticText_layer_info = wx.StaticText( sortingSizer1.GetStaticBox(), wx.ID_ANY, u"Note! Layers at the top of the list will be drawn on top of layers further down.", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText_layer_info.Wrap( -1 )

        bSizer512.Add( self.m_staticText_layer_info, 0, wx.ALL, 5 )


        sortingSizer1.Add( bSizer512, 1, wx.EXPAND, 5 )


        m_sizer_template.Add( sortingSizer1, 1, wx.ALL|wx.EXPAND, 5 )

        layersSizer1 = wx.StaticBoxSizer( wx.StaticBox( sbSizer23.GetStaticBox(), wx.ID_ANY, u"Disabled Layers" ), wx.VERTICAL )

        bSizer411 = wx.BoxSizer( wx.HORIZONTAL )

        bSizer611 = wx.BoxSizer( wx.VERTICAL )

        disabledLayersSortOrderBoxChoices = []
        self.disabledLayersSortOrderBox = wx.ListBox( layersSizer1.GetStaticBox(), wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, disabledLayersSortOrderBoxChoices, wx.LB_SINGLE|wx.LB_SORT|wx.BORDER_SIMPLE )
        self.disabledLayersSortOrderBox.SetToolTip( u"Double-click to enable layer" )

        bSizer611.Add( self.disabledLayersSortOrderBox, 1, wx.ALL|wx.EXPAND, 5 )


        bSizer411.Add( bSizer611, 1, wx.EXPAND, 5 )

        bSizer511 = wx.BoxSizer( wx.VERTICAL )

        self.m_button_layer_enable = wx.Button( layersSizer1.GetStaticBox(), wx.ID_ANY, u"Enable", wx.DefaultPosition, wx.Size( 70,30 ), 0|wx.BORDER_DEFAULT )

        self.m_button_layer_enable.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_PLUS, wx.ART_BUTTON ) )
        self.m_button_layer_enable.SetBitmapMargins( wx.Size( 2,-1 ) )
        bSizer511.Add( self.m_button_layer_enable, 0, wx.ALL, 5 )


        bSizer411.Add( bSizer511, 0, wx.ALIGN_RIGHT, 5 )


        layersSizer1.Add( bSizer411, 1, wx.EXPAND, 5 )


        m_sizer_template.Add( layersSizer1, 1, wx.ALL|wx.EXPAND, 5 )


        sbSizer23.Add( m_sizer_template, 1, wx.EXPAND, 5 )


        bSizer301.Add( sbSizer23, 1, wx.EXPAND, 5 )


        bSizer20.Add( bSizer301, 1, wx.EXPAND, 5 )


        self.SetSizer( bSizer20 )
        self.Layout()

        # Connect Events
        self.templatesSortOrderBox.Bind( wx.EVT_LISTBOX, self.OnTemplateEdit )
        self.templatesSortOrderBox.Bind( wx.EVT_LISTBOX_DCLICK, self.OnTemplateDisable )
        self.m_button_template_up.Bind( wx.EVT_BUTTON, self.OnTemplateSortOrderUp )
        self.m_button_template_down.Bind( wx.EVT_BUTTON, self.OnTemplateSortOrderDown )
        self.m_button_template_new.Bind( wx.EVT_BUTTON, self.OnTemplateNew )
        self.m_button_template_copy.Bind( wx.EVT_BUTTON, self.OnTemplateClone )
        self.m_button_template_disable.Bind( wx.EVT_BUTTON, self.OnTemplateDisable )
        self.disabledTemplatesSortOrderBox.Bind( wx.EVT_LISTBOX_DCLICK, self.OnTemplateEnable )
        self.m_button_template_enable.Bind( wx.EVT_BUTTON, self.OnTemplateEnable )
        self.m_button4.Bind( wx.EVT_BUTTON, self.OnTemplateDelete )
        self.saveSettingsBtn.Bind( wx.EVT_BUTTON, self.OnSaveSettings )
        self.loadSettingsBtn.Bind( wx.EVT_BUTTON, self.OnLoadSettings )
        self.m_button42.Bind( wx.EVT_BUTTON, self.OnPerform )
        self.m_button43.Bind( wx.EVT_BUTTON, self.OnExit )
        self.m_textCtrl_template_name.Bind( wx.EVT_TEXT, self.OnTemplateNameChange )
        self.m_textCtrl_color.Bind( wx.EVT_TEXT, self.OnSaveLayer )
        self.m_button_pick_color.Bind( wx.EVT_BUTTON, self.OnPickColor )
        self.m_checkBox_negative.Bind( wx.EVT_CHECKBOX, self.OnSaveLayer )
        self.m_checkBox_reference_designators.Bind( wx.EVT_CHECKBOX, self.OnSaveLayer )
        self.m_checkBox_footprint_values.Bind( wx.EVT_CHECKBOX, self.OnSaveLayer )
        self.layersSortOrderBox.Bind( wx.EVT_LISTBOX, self.OnLayerEdit )
        self.layersSortOrderBox.Bind( wx.EVT_LISTBOX_DCLICK, self.OnLayerDisable )
        self.m_button_layer_up.Bind( wx.EVT_BUTTON, self.OnLayerSortOrderUp )
        self.m_button_layer_down.Bind( wx.EVT_BUTTON, self.OnLayerSortOrderDown )
        self.m_button_layer_disable.Bind( wx.EVT_BUTTON, self.OnLayerDisable )
        self.disabledLayersSortOrderBox.Bind( wx.EVT_LISTBOX_DCLICK, self.OnLayerEnable )
        self.m_button_layer_enable.Bind( wx.EVT_BUTTON, self.OnLayerEnable )

    def __del__( self ):
        pass


    # Virtual event handlers, override them in your derived class
    def OnTemplateEdit( self, event ):
        event.Skip()

    def OnTemplateDisable( self, event ):
        event.Skip()

    def OnTemplateSortOrderUp( self, event ):
        event.Skip()

    def OnTemplateSortOrderDown( self, event ):
        event.Skip()

    def OnTemplateNew( self, event ):
        event.Skip()

    def OnTemplateClone( self, event ):
        event.Skip()


    def OnTemplateEnable( self, event ):
        event.Skip()


    def OnTemplateDelete( self, event ):
        event.Skip()

    def OnSaveSettings( self, event ):
        event.Skip()

    def OnLoadSettings( self, event ):
        event.Skip()

    def OnPerform( self, event ):
        event.Skip()

    def OnExit( self, event ):
        event.Skip()

    def OnTemplateNameChange( self, event ):
        event.Skip()

    def OnSaveLayer( self, event ):
        event.Skip()

    def OnPickColor( self, event ):
        event.Skip()




    def OnLayerEdit( self, event ):
        event.Skip()

    def OnLayerDisable( self, event ):
        event.Skip()

    def OnLayerSortOrderUp( self, event ):
        event.Skip()

    def OnLayerSortOrderDown( self, event ):
        event.Skip()


    def OnLayerEnable( self, event ):
        event.Skip()






import os
import shutil
from pathlib import Path
import json
import re

plugin_src_path = (Path(__file__).parent.parent / "src").absolute()
resources_path = Path(__file__).parent / "resources"
repo_path = (Path(__file__).parent.parent.parent).absolute()
board2pdf_src_path = repo_path / "src" / "board2pdf"

build_path = Path("build").absolute()

version_path = plugin_src_path / "_version.py"
metadata_template = Path(__file__).parent / "metadata_template.json"

# Delete build and recreate
try:
    shutil.rmtree(build_path)
except FileNotFoundError:
    pass
os.makedirs(build_path / "plugin")
os.makedirs(build_path / "plugin" / "resources")

# Copy plugin. Sym-links doesn't work on Windows, that would've been easier
shutil.copytree(plugin_src_path / "dialog", build_path / "plugin" / "plugins" / "dialog")
shutil.copy(plugin_src_path / "__init__.py", build_path / "plugin" / "plugins" / "__init__.py")
shutil.copy(plugin_src_path / "board2pdf_action.py", build_path / "plugin" / "plugins" / "board2pdf_action.py")
shutil.copy(resources_path / "icon.png", build_path / "plugin" / "plugins" / "icon.png")
shutil.copy(resources_path / "icon-64x64.png", build_path / "plugin" / "resources" / "icon.png")
shutil.copy(repo_path / "README.md", build_path / "plugin" / "plugins" / "README.md")
shutil.copy(repo_path / "LICENSE", build_path / "plugin" / "plugins" / "LICENSE")
shutil.copytree(board2pdf_src_path / "pypdf", build_path / "plugin" / "plugins" / "pypdf")
shutil.copy(board2pdf_src_path / "cli.py", build_path / "plugin" / "plugins" / "board2pdf-cli.py")
shutil.copy(board2pdf_src_path / "default_config.ini", build_path / "plugin" / "plugins" / "default_config.ini")
shutil.copy(board2pdf_src_path / "persistence.py", build_path / "plugin" / "plugins" / "persistence.py")
shutil.copy(board2pdf_src_path / "plot.py", build_path / "plugin" / "plugins" / "plot.py")
shutil.copy(version_path, build_path / "plugin" / "plugins" / "_version.py")

# Clean out any __pycache__ or .pyc files (https://stackoverflow.com/a/41386937)
[p.unlink() for p in build_path.rglob("*.py[co]")]
[p.rmdir() for p in build_path.rglob("__pycache__")]

# Don't include test_dialog.py. It isn't needed. It's a developer thing.
[p.unlink() for p in build_path.rglob("test_dialog.py")]

# Copy metadata
metadata = build_path / "plugin" / "metadata.json"
shutil.copy(metadata_template, metadata)

# Load up json script
with metadata.open("r") as f:
    md = json.load(f)

# Get version from resource/_version.py
# https://stackoverflow.com/a/7071358

verstrline = version_path.open("rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    verstr = "1.0.0"

# Update download URL
md["versions"][0].update(
    {
        "version": verstr,
        "download_url": "https://gitlab.com/dennevi/kicad-plugins-releases/-/raw/main/Board2Pdf/releases/Board2Pdf_v{0}.zip".format(
            verstr
        ),
    }
)

# Update metadata.json
with metadata.open("w") as of:
    json.dump(md, of, indent=2)

# Zip all files

pcm_name = "releases/Board2Pdf_v{0}".format(md["versions"][0]["version"])
pcm_path = build_path / pcm_name
zip_file = shutil.make_archive(pcm_path, "zip", build_path / "plugin")

# Rename the plugin directory so we can upload it as an artifact - and avoid the double-zip
shutil.move(build_path / "plugin", pcm_path)



# from __future__ import absolute_import

import os
import shutil
import sys
import pcbnew
import wx


if __name__ == "__main__":
    # Circumvent the "scripts can't do relative imports because they are not
    # packages" restriction by asserting dominance and making it a package!
    dirname = os.path.dirname(os.path.abspath(__file__))
    __package__ = os.path.basename(dirname)
    sys.path.insert(0, os.path.dirname(dirname))
    __import__(__package__)

from . _version import __version__
from . import plot
from . import dialog
from . import persistence


_board = None
def get_board():
    global _board
    if _board is None:
        _board = pcbnew.GetBoard()
    return _board


def run_with_dialog():
    board = get_board()
    pcb_file_name = board.GetFileName()
    board2pdf_dir = os.path.dirname(os.path.abspath(__file__))
    pcb_file_dir = os.path.dirname(os.path.abspath(pcb_file_name))
    default_configfile = os.path.join(board2pdf_dir, "default_config.ini")
    global_configfile = os.path.join(board2pdf_dir, "board2pdf.config.ini")
    local_configfile = os.path.join(pcb_file_dir, "board2pdf.config.ini")

    # Not sure it this is needed any more.
    if not pcb_file_name:
        wx.MessageBox('Please save the board file before plotting the pdf.')
        return

    # If local config.ini file doesn't exist, use global. If global doesn't exist, use default.
    if os.path.exists(local_configfile):
        configfile = local_configfile
        configfile_name = "local"
    elif os.path.exists(global_configfile):
        configfile = global_configfile
        configfile_name = "global"
    else:
        configfile = default_configfile
        configfile_name = "default"

    config = persistence.Persistence(configfile)
    config.load()
    config.default_settings_file_path = default_configfile
    config.global_settings_file_path = global_configfile
    config.local_settings_file_path = local_configfile

    def perform_export(dialog_panel):
        plot.plot_pdfs(board, dialog_panel.outputDirPicker.Path, config.templates,
                          dialog_panel.templatesSortOrderBox.GetItems(),
                          dialog_panel.m_checkBox_delete_temp_files.IsChecked(),
                          dialog_panel.m_checkBox_create_svg.IsChecked(),
                          dialog_panel.m_checkBox_delete_single_page_files.IsChecked(), dialog_panel,
                          layer_scale=config.layer_scale,
                          assembly_file_extension=config.assembly_file_extension)
        dialog_panel.m_progress.SetValue(100)
        dialog_panel.Refresh()
        dialog_panel.Update()

    def load_saved(dialog_panel, config):
        # Update dialog with data from saved config.
        dialog_panel.outputDirPicker.Path = config.output_path
        dialog_panel.templatesSortOrderBox.SetItems(config.enabled_templates)
        dialog_panel.disabledTemplatesSortOrderBox.SetItems(config.disabled_templates)
        dialog_panel.m_checkBox_delete_temp_files.SetValue(config.del_temp_files)
        dialog_panel.m_checkBox_create_svg.SetValue(config.create_svg)
        dialog_panel.m_checkBox_delete_single_page_files.SetValue(config.del_single_page_files)
        dialog_panel.ClearTemplateSettings()
        dialog_panel.hide_template_settings()

    dlg = dialog.SettingsDialog(config, perform_export, load_saved, __version__)
    try:
        icon_path = os.path.join(os.path.dirname(__file__), 'icon.png')
        icon = wx.Icon(icon_path)
        dlg.SetIcon(icon)
    except Exception:
        pass

    # Update dialog with data from saved config.
    dlg.panel.outputDirPicker.Path = config.output_path
    dlg.panel.templatesSortOrderBox.SetItems(config.enabled_templates)
    dlg.panel.disabledTemplatesSortOrderBox.SetItems(config.disabled_templates)
    dlg.panel.m_checkBox_delete_temp_files.SetValue(config.del_temp_files)
    dlg.panel.m_checkBox_create_svg.SetValue(config.create_svg)
    dlg.panel.m_checkBox_delete_single_page_files.SetValue(config.del_single_page_files)
    dlg.panel.m_staticText_status.SetLabel(f'Status: loaded {configfile_name} settings')

    # Check if able to import PyMuPDF.
    has_pymupdf = True
    try:
        import pymupdf  # This imports PyMuPDF

    except:
        try:
            import fitz as pymupdf  # This imports PyMuPDF using old name

        except:
            try:
                import fitz_old as pymupdf # This imports PyMuPDF using temporary old name

            except:
                has_pymupdf = False

    # after pip uninstall PyMuPDF the import still works, but not `open()`
    # check if it's possible to call pymupdf.open()
    if has_pymupdf:
        try:
            pymupdf.open()
        except:
            has_pymupdf = False

    # If it was possible to import and open PyMuPdf, select pymupdf otherwise select pypdf.
    if has_pymupdf:
        dlg.panel.m_radio_pymupdf.SetValue(True)
        dlg.panel.m_radio_merge_pymupdf.SetValue(True)
    else:
        dlg.panel.m_radio_pypdf.SetValue(True)
        dlg.panel.m_radio_merge_pypdf.SetValue(True)

    dlg.ShowModal()
    # response = dlg.ShowModal()
    # if response == wx.ID_CANCEL:

    dlg.Destroy()


class board2pdf(pcbnew.ActionPlugin):
    def defaults(self):
        self.name = f"Board2Pdf\nversion {__version__}"
        self.category = "Plot"
        self.description = "Plot pcb to pdf."
        self.show_toolbar_button = True  # Optional, defaults to False
        self.icon_file_name = os.path.join(os.path.dirname(__file__), 'icon.png')  # Optional

    def Run(self):
        run_with_dialog()


if __name__ == "__main__":
    _board = pcbnew.LoadBoard(sys.argv[1])
    #run_with_dialog()
    app = wx.App()
    p = board2pdf()
    p.Run()
